@@ -19,6 +19,7 @@
 
 
 import sys, os
+import pkg_resources
 
 # Add and use Pylons theme
 if 'sphinx-build' in ' '.join(sys.argv): # protect against dumb importers
@@ -83,7 +84,7 @@ copyright = '2011, Agendaless Consulting <chrism@plope.com>'
 # other places throughout the built documents.
 #
 # The short X.Y version.
-version = '0.4'
+version = pkg_resources.get_distribution('pyramid_zodbconn').version
 # The full version, including alpha/beta/rc tags.
 release = version
 
